@@ -50,7 +50,7 @@ class TestLoss(unittest.TestCase):
         masked_lm_loss = masked_lm_loss.view(-1, len(targets))  # view by batch size
         masked_lm_loss = masked_lm_loss.sum(dim=0)
         masked_lm_loss = masked_lm_loss.mean()
-        print(masked_lm_loss, norm_loss_fct(outputs, targets))
+        print(masked_lm_loss.mean(), norm_loss_fct(outputs, targets).mean())
 
     def testNegativeCElLoss(self):
         outputs = torch.Tensor([[0.00000000000009, 5, 0.5], [0.00000000000000000001, 69, 9]])
@@ -82,6 +82,41 @@ class TestLoss(unittest.TestCase):
         self.assertTrue(criterion(self.outputs, self.onen_targets) > custom_criterion(self.outputs, self.onen_targets))
 
 
+class TestTok(unittest.TestCase):
+
+    def testTok(self):
+        tokenizer = BertTokenizer.from_pretrained('voidful/albert_chinese_tiny')
+        begin = tfkit.utility.tok.tok_begin(tokenizer)
+        self.assertEqual(begin, "[CLS]")
+        sep = tfkit.utility.tok.tok_sep(tokenizer)
+        self.assertEqual(sep, "[SEP]")
+        mask = tfkit.utility.tok.tok_mask(tokenizer)
+        self.assertEqual(mask, "[MASK]")
+        pad = tfkit.utility.tok.tok_pad(tokenizer)
+        self.assertEqual(pad, "[PAD]")
+
+    def testGetXUnkToken(self):
+        tokenizer = BertTokenizer.from_pretrained('voidful/albert_chinese_tiny')
+        result = tfkit.utility.tok.get_topP_unk_token(tokenizer, file_paths=[], topP=0.5)
+        self.assertFalse(result)
+        result = tfkit.utility.tok.get_freqK_unk_token(tokenizer, file_paths=[], freqK=10)
+        self.assertFalse(result)
+
+    def testHandleExceed(self):
+        tokenizer = BertTokenizer.from_pretrained('voidful/albert_chinese_tiny')
+        seq = " ".join([str(_) for _ in range(100)])
+        maxlen = 50
+        for mode in ['remove', 'slide', 'start_slice', 'end_slice']:
+            rlt,_ = tfkit.utility.tok.handle_exceed(tokenizer, seq, maxlen, mode=mode)
+            print(mode, len(rlt))
+            if mode == 'remove':
+                self.assertTrue(len(rlt) == 0)
+            if mode == 'slide':
+                self.assertTrue(len(rlt) > 1)
+            for i in rlt:
+                self.assertTrue(len(i) == 50)
+
+
 class TestEval(unittest.TestCase):
     def testEMF1(self):
         tokenizer = BertTokenizer.from_pretrained('voidful/albert_chinese_tiny')
@@ -144,6 +179,7 @@ class TestEval(unittest.TestCase):
     @pytest.mark.skip()
     def testNLG(self):
         tokenizer = BertTokenizer.from_pretrained('voidful/albert_chinese_tiny')
+
         eval = tfkit.utility.eval_metric.EvalMetric(tokenizer)
         eval.add_record("input", "a b c", "a b c[SEP]a c c[SEP]", task='default')
         for s in eval.cal_score('nlg'):
@@ -154,6 +190,13 @@ class TestEval(unittest.TestCase):
         for s1 in eval1.cal_score('nlg'):
             print(s1)
 
+        eval1 = tfkit.utility.eval_metric.EvalMetric(tokenizer, max_candidate=1)
+        eval1.add_record("input", "abb", " abc ", task='default')
+        eval1.add_record("input", "abc", " abc ", task='default')
+        eval1.add_record("input", "abd", " abc ", task='default')
+        for s1 in eval1.cal_score('nlg'):
+            print(s1)
+
         eval3 = tfkit.utility.eval_metric.EvalMetric(tokenizer, max_candidate=3)
         eval3.add_record("input", "abc ", "abb [SEP]acc[SEP] abc ", task='default')
         for s3 in eval3.cal_score('nlg'):
@@ -163,7 +206,7 @@ class TestEval(unittest.TestCase):
         eval6.add_record("input", "abc", "abb [SEP] acc [SEP]abc", task='default')
         for s6 in eval6.cal_score('nlg'):
             print(s6)
-        self.assertTrue(s1 == s3 == s6)
+        self.assertTrue(s1[0] == s3[0] == s6[0])
 
         eval1 = tfkit.utility.eval_metric.EvalMetric(tokenizer, max_candidate=1)
         eval1.add_record("input", "opq", "abc", task='default')
@@ -179,7 +222,7 @@ class TestEval(unittest.TestCase):
         eval6.add_record("input", "opq", "abb [SEP] acc[SEP]abc", task='default')
         for s6 in eval6.cal_score('nlg'):
             print(s6)
-        self.assertTrue(s1 == s3 == s6)
+        self.assertTrue(s1[0] == s3[0] == s6[0])
 
     def testClassify(self):
         tokenizer = BertTokenizer.from_pretrained('voidful/albert_chinese_tiny')
@@ -247,6 +290,7 @@ class TestEval(unittest.TestCase):
         for s in eval.cal_score('classification'):
             print(s[0])
             print(s[1])
+            print(s)
 
         eval = tfkit.utility.eval_metric.EvalMetric(tokenizer)
         eval.add_record("input", '攝影台', ['攝影台'], task='default')
