@@ -5,6 +5,7 @@ from torch import nn
 import torch.nn.functional as F
 from torch.autograd import Variable
 import numpy as np
+from math import log, e
 
 
 class BCEFocalLoss(nn.Module):
@@ -31,7 +32,7 @@ class BCEGWLoss(nn.Module):
     def forward(self, input, target):
         BCE_loss = F.binary_cross_entropy_with_logits(input, target, reduction='none')
         BCE_loss = BCE_loss.view(-1)
-        pt = sigmoid(BCE_loss)  # prevents nans when probability 0
+        pt = F.sigmoid(BCE_loss)  # prevents nans when probability 0
         loss = (self.gaussian(pt, variance=0.1 * math.exp(1), mean=0.5) - 0.1 * pt) * BCE_loss
         return loss.mean()
 
@@ -116,56 +117,3 @@ class LabelSmoothingLoss(nn.Module):
             return torch.mean(torch.sum(-true_dist * pred, dim=self.dim))
         else:
             return torch.sum(-true_dist * pred, dim=self.dim)
-
-
-class LossDropper(nn.Module):
-    """From 'Improved Natural Language Generation via Loss Truncation'"""
-
-    def __init__(
-            self,
-            dropc=0.4,
-            min_count=10000,
-            recompute=10000,
-            verbose=True
-    ):
-        super().__init__()
-        self.keepc = 1. - dropc
-        self.count = 0
-        self.min_count = min_count
-
-        self.recompute = recompute
-        self.last_computed = 0
-        self.percentile_val = 100000000.
-        self.cur_idx = 0
-
-        self.verbose = verbose
-
-        self.vals = np.zeros(self.recompute, dtype=np.float32)
-
-    def forward(self, loss):
-        if loss is None:
-            return loss
-
-        self.last_computed += loss.numel()
-        self.count += loss.numel()
-        if self.count < len(self.vals):
-            self.vals[self.count - loss.numel():self.count] = loss.detach().cpu().numpy().flatten()
-            self.cur_idx += loss.numel()
-            return (loss < np.inf).type(loss.dtype)
-        else:
-            for idx, item in enumerate(loss):
-                self.vals[self.cur_idx] = item
-                self.cur_idx += 1
-                if self.cur_idx >= len(self.vals):
-                    self.cur_idx = 0
-        if self.count < self.min_count:
-            return (loss < np.inf).type(loss.dtype)
-
-        if self.last_computed > self.recompute:
-            self.percentile_val = np.percentile(self.vals, self.keepc * 100)
-            if self.verbose:
-                print('Using cutoff', self.percentile_val)
-            self.last_computed = 0
-
-        mask = (loss < self.percentile_val).type(loss.dtype)
-        return mask
